using System;
using System.Windows.Forms;
using System.Drawing;

class FunWithTheMouse : Form
{
    // Declare some objects we'll want to talk to from a few methods

    PictureBox pictureBox1;
    Label label1;
    Point spotClicked;
    
    // The constructor method for our class
    public FunWithTheMouse()
    {
        // Set the window size

        this.Size = new Size(640, 480);

        // Load a photo into a picture box and add it to the form
        
        pictureBox1 = new PictureBox();
        pictureBox1.Image = (Image)new Bitmap(@"../../images/foal.jpg");
        pictureBox1.SizeMode = PictureBoxSizeMode.Normal;
        pictureBox1.Dock = DockStyle.Fill;
        this.Controls.Add(pictureBox1);

        // Add a label to the bottom, carrying some instructions

        label1 = new Label();
        label1.BackColor = Color.Wheat;
        label1.Dock = DockStyle.Bottom;
        label1.Text = 
            "Drag rectangle with left mouse button (draw) or another " +
            "mouse button (lighten). To scribble, hold SHIFT and move mouse.";
        label1.TextAlign = ContentAlignment.MiddleCenter;
        this.Controls.Add(label1);

        // Hook up the picturebox to event handlers for doing mousy things

        this.pictureBox1.MouseDown += new MouseEventHandler(MouseButtonIsDown);
        this.pictureBox1.MouseUp += new MouseEventHandler(MouseButtonIsUp);
        this.pictureBox1.MouseMove += new MouseEventHandler(TheMouseMoved);
    }

    // An event handler that will run when the mouse is MOVED
    public void TheMouseMoved(object sender, MouseEventArgs e)
    {
        // If the user is holding down a SHIFT key on the keyboard
        if ((Control.ModifierKeys & Keys.Shift) == Keys.Shift)
        {
            // Prepare a drawing surface on the picture
            System.Drawing.Graphics g = this.pictureBox1.CreateGraphics();

            // Use a yellow pen
            System.Drawing.Pen yellowPen = new System.Drawing.Pen(Color.Yellow, 3);

            // Draw a circle (an ellipse with equal width & height)
            // at the current X and Y co-ordinates of the mouse
            g.DrawEllipse(yellowPen, e.X, e.Y, 40, 40);

            // Clean up
            g.Dispose();
        }
    }

    // An event handler that will run when a mouse button is pressed DOWN
    public void MouseButtonIsDown(object sender, MouseEventArgs e)
    {
        // Just remember WHERE the mouse was clicked. When the
        // button is released, we'll want to retrieve this location
 
        spotClicked.X = e.X;    // horizontal co-ordinate
        spotClicked.Y = e.Y;    // vertical co-ordinate
    }

    // An event handler that will run when a mouse button is released
    public void MouseButtonIsUp(object sender, MouseEventArgs e)
    {
        /* Ah, the user let go the button they were holding down! */

        // Define a rectangle (not yet a visual one) that describes
        // the area of the picture the user wants to do something with

        Rectangle r = new Rectangle();
        
        // The left, top corner is where they pressed the mouse down,
        // which we "remembered" in the method above

        r.X = spotClicked.X;
        r.Y = spotClicked.Y;
        
        // The width and height of the rectangle can be worked out
        // by subtracting the original point clicked from where
        // the mouse is now (when the user is letting go)

        r.Width = e.X - spotClicked.X;
        r.Height = e.Y - spotClicked.Y;

        if (e.Button == MouseButtons.Left)
        {
            /* If left button was pressed/released, just draw 
               a visual rectangle so they can see it */

            // Prepare a drawing surface on the picture box

            Graphics g = this.pictureBox1.CreateGraphics();

            // Draw a rectangle in red

            Pen redPen = new Pen(Color.Red, 2);
            g.DrawRectangle(redPen, r);
        }
        else
        {
            //  If it was another button, call our more complicated 
            //  method that knows how to "lighten" an area of the photo

            ChangeLightness(r);
        }
    }

    // A method that increases the lightness of the image portion 
    // selected, by increasing the lightness of each pixel in that area
    public void ChangeLightness(Rectangle rect)
    {
        int newRed, newGreen, newBlue;
        Color pixel;

        // Get a copy of the picture that's in the picture box

        System.Drawing.Bitmap picture = new Bitmap(this.pictureBox1.Image);

        // Since brightening an area can take a long time,
        // warn the user about this if they select a big area

        if ( (rect.Width>150) || (rect.Height>150 ) )
        {
            
            DialogResult result = 
                 MessageBox.Show
                 (
                    "The area you selected is large and may take a long time to lighten",
                    "Warning",
                    MessageBoxButtons.OKCancel
                 );
                
            // If they click "cancel", jump out of this method
            // and return to whatever called it
            if ( result == DialogResult.Cancel ) return;
        }

        /* Step through all pixels in the given block, doubling the 
            brightness of each pixel's RGB (red, green and blue) colors */
        
        // From left to right, across the width of the area selected ...
        
        for (int x = rect.X; x < rect.X + rect.Width; x++)
        {
            // And from top to bottom, down the height of the area selected ...
            
            for (int y = rect.Y; y < (rect.Y + rect.Height); y++)
            {
                // Read the pixel we're at
                
                pixel = picture.GetPixel(x, y);

                // Prepare a brighter version of the three primary colors in that pixel

                newRed = (int)Math.Round(pixel.R * 2.0, 0);
                if (newRed > 255) newRed = 255;
                newGreen = (int)Math.Round(pixel.G * 2.0, 0);
                if (newGreen > 255) newGreen = 255;
                newBlue = (int)Math.Round(pixel.B * 2.0, 0);
                if (newBlue > 255) newBlue = 255;

                // Apply the new colors to the pixel

                picture.SetPixel
                (
                    x, 
                    y, 
                    Color.FromArgb((byte)newRed,(byte)newGreen, (byte)newBlue)
                 );
            }
        }

        // Put the modified copy of the picture into the pictureBox 
        // so the user can see the changes

        this.pictureBox1.Image = picture;

    }

    static void Main()
    {
        // Create a new instance of our form class

        Application.Run(new FunWithTheMouse());
    }
}